@@ -14,6 +14,8 @@
 #include <KIconLoader>
 
 #include <KColorScheme>
+#include <KConfigGroup>
+#include <QDBusConnection>
 
 class IconLoaderSingleton
 {
@@ -43,6 +45,14 @@ public:
     {
         connect(qGuiApp, &QGuiApplication::paletteChanged,
                 this, &StyleSingleton::refresh);
+
+        // Use DBus in order to listen for kdeglobals changes directly, as the
+        // QApplication doesn't expose the font variants we're looking for,
+        // namely smallFont.
+        QDBusConnection::sessionBus().connect( QString(),
+        QStringLiteral( "/KGlobalSettings" ),
+        QStringLiteral( "org.kde.KGlobalSettings" ),
+        QStringLiteral( "notifyChange" ), this, SIGNAL(configurationChanged()));
     }
 
     void refresh()
@@ -81,6 +91,9 @@ public:
         case PlatformTheme::Complementary:
             set = KColorScheme::ColorSet::Complementary;
             break;
+        case PlatformTheme::Header:
+            set = KColorScheme::ColorSet::Header;
+            break;
         case PlatformTheme::Window:
         default:
             set = KColorScheme::ColorSet::Window;
@@ -120,6 +133,7 @@ public:
     KColorScheme viewScheme;
 
 Q_SIGNALS:
+    void configurationChanged();
     void paletteChanged();
 
 private:
@@ -158,6 +172,7 @@ PlasmaDesktopTheme::PlasmaDesktopTheme(QObject *parent)
 
     //TODO: correct? depends from https://codereview.qt-project.org/206889
     connect(qGuiApp, &QGuiApplication::fontDatabaseChanged, this, [this]() {setDefaultFont(qApp->font());});
+    configurationChanged();
 
     connect(this, &PlasmaDesktopTheme::colorSetChanged,
             this, &PlasmaDesktopTheme::syncColors);
@@ -167,11 +182,29 @@ PlasmaDesktopTheme::PlasmaDesktopTheme(QObject *parent)
     connect(s_style->data(), &StyleSingleton::paletteChanged,
             this, &PlasmaDesktopTheme::syncColors);
 
+    connect(s_style->data(), &StyleSingleton::configurationChanged,
+            this, &PlasmaDesktopTheme::configurationChanged);
+
     syncColors();
 }
 
 PlasmaDesktopTheme::~PlasmaDesktopTheme() = default;
 
+void PlasmaDesktopTheme::configurationChanged()
+{
+    KSharedConfigPtr ptr = KSharedConfig::openConfig();
+    KConfigGroup general( ptr->group("general") );
+    setSmallFont(general.readEntry("smallestReadableFont", []() {
+        auto smallFont = qApp->font();
+        if (smallFont.pixelSize() != -1) {
+            smallFont.setPixelSize(smallFont.pixelSize()-2);
+        } else {
+            smallFont.setPointSize(smallFont.pointSize()-2);
+        }
+        return smallFont;
+    }()));
+}
+
 QIcon PlasmaDesktopTheme::iconFromTheme(const QString &name, const QColor &customColor)
 {
     QPalette pal = palette();
